#include "sudoku.hpp"
#include <sstream>
#include <iostream>
#include <string>

/**
   Renvoie vrai si la ligne ne contient aucun chiffre ou commence par #
*/
bool ligne_vide(const string & s) {
  if (s.length() > 0 && *(s.cbegin()) == '#') {
    return true;
  }
  for(auto it = s.cbegin(); it != s.cend(); ++it) {
    if (*it >= '0' && *it <= '9') {
      return false;
    }
  }
  return true;
}

bool lit_ligne_sudoku(const string & ligne, vector<int> & ligne_grille) {
  bool lu_entiers = true;
  bool lit_entier = false;
  string buf = "";
  for(auto it = ligne.cbegin(); it != ligne.cend(); ++it) {
    char c = *it;
    if (c == '#') {
      break;
    } else if ('0' <= c && c <= '9') {
      buf += c;
      lit_entier = true;
    } else {
      if (buf.size() > 0) {
	ligne_grille.push_back(stoi(buf));
	lu_entiers = true;
	lit_entier = false;
	buf = "";
      }
    }
  }
  if (buf.size() > 0) {
    ligne_grille.push_back(stoi(buf));
  }
  return lu_entiers;
}

void lit_sudoku(istream& input, sudoku_t & sudoku) {
  string ligne;
  while (input.good()) {
    getline(input,ligne);
    if (!ligne_vide(ligne)) {
      vector<int> s_line;
      if (lit_ligne_sudoku(ligne, s_line)) {
	sudoku.push_back(s_line);
      }
    }
  }
}


ostream& operator<<(ostream& out, const vector<int>& ligne_grille) {
  out << " { ";
  bool not_first = false;
  for(auto it = ligne_grille.cbegin(); it != ligne_grille.cend(); ++it) {
    out << *it; 
    if (not_first) {
      out << ",";
      not_first = true;
    }
    out << " ";
  }
  out << "} ";
  return out;
}


void ecrit_sudoku(ostream& output, const sudoku_t & sudoku) {
  for(auto it_l = sudoku.cbegin(); it_l != sudoku.cend(); ++it_l) {
    for(auto it_c = it_l->cbegin(); it_c != it_l->cend(); ++it_c) {
      output << *it_c << " ";
    }
    output << endl << endl;
  }
}

void genere_probleme_sat(ostream& output, const sudoku_t& sudoku) {
  // TODO: à implémenter
}
